/* 
 * File:   communication.h
 * Author: Carlo Alessandro Nicolau @ INFN
 * 
 * Firmware for BPD board.
 * Communication functions.
 * 
 */

#ifndef __COMMUNICATION_H
#define	__COMMUNICATION_H

#include "typedefs.h"
#include "errorcodes.h"

// error definitions
#define ERR_COMMUNICATION_PENDINGCMD    ((err_t)(ERR_BASE_COMMUNICATION+0))  // something was received before managing the previous command
#define ERR_COMMUNICATION_WRONG_SOF     ((err_t)(ERR_BASE_COMMUNICATION+1))  // error while waiting for SOF
#define ERR_COMMUNICATION_WRONG_ADDR    ((err_t)(ERR_BASE_COMMUNICATION+2))  // error while waiting for ADDR
#define ERR_COMMUNICATION_ASCII2HEX     ((err_t)(ERR_BASE_COMMUNICATION+3))  // wrong ascii to hex value conversion requested
#define ERR_COMMUNICATION_RECVBUFF_FULL ((err_t)(ERR_BASE_COMMUNICATION+4))  // receive data buffer full
#define ERR_COMMUNICATION_CHECKSUM      ((err_t)(ERR_BASE_COMMUNICATION+5))  // checksum error
#define ERR_COMMUNICATION_WRONG_EOF     ((err_t)(ERR_BASE_COMMUNICATION+6))  // error while waiting for EOF
#define ERR_COMMUNICATION_UNKNWOWN      ((err_t)(ERR_BASE_COMMUNICATION+7))  // the received packet was in an unknown condition
#define ERR_COMMUNICATION_EXEC          ((err_t)(ERR_BASE_COMMUNICATION+8))  // something went wrong during command execution

// defines related to the communication protocol
#define COMMUNICATION_PACKET_SOF        0x01          // start of frame value
#define COMMUNICATION_PACKET_ADDR       0x30          // address (...yes, only this address is listened to)
#define COMMUNICATION_PACKET_EOF        0x00          // end of frame value

// type definition for communication parse state machine status
typedef enum {
    WAITING_SOF = 0,    // waiting for a start of frame byte
    WAITING_ADDR,       // waiting for the address byte
    WAITING_CMD,        // waiting for the command code
    WAITING_DATA_EVEN,  // waiting for the command data bytes even (0,2,4,...) nibbles
    WAITING_DATA_ODD,   // waiting for the command data bytes odd (1,3,5,...) nibbles
    WAITING_CHK,        // waiting for the check code
    WAITING_EOF,        // waiting for the end-of-frame
    COMMAND_READY       // a command is ready
} communication_parser_state_t;

// communication parser functions
void communication_parser_reset(void);
void communication_parser_appendbyte(uint8_t _byte);
bool communication_parser_commandready(void);
err_t communication_parser_executecommand(void);
void communication_transmitter_sendresponse(void);

// function to read and write payload data, used inside the command callbacks
void communication_request_payload_retriever_reset(void);
uint8_t communication_request_payload_retrieve_uint8_t(void);
uint16_t communication_request_payload_retrieve_uint16_t(void);
void communication_response_payload_appender_reset(void);
void communication_response_payload_append_uint8_t(uint8_t _value);
void communication_response_payload_append_uint16_t(uint16_t _value);
void communication_response_payload_append_uint32_t(uint32_t _value);

#endif// __COMMUNICATION_H

