/* 
 * File:   rescue.h
 * Author: Carlo Alessandro Nicolau @ INFN
 * 
 * Firmware for BPD board.
 * Rescue functions.
 */

#include "rescue.h"
#include "configuration.h"
#include "timing.h"
#include "generated/sources/variables.h"

// value of the timeout for the rescue procedure
#if RESCUE_TIMEOUT_SECONDS > 0xFFFF
    #error "RESCUE_TIMEOUT_SECONDS too big for uint16_t"
#endif
uint16_t _rescue_timeout_seconds = (uint16_t)RESCUE_TIMEOUT_SECONDS;

// rescue enable status
bool _rescue_enabled = true;

// current rescue status: fired or not
bool _rescue_fired = false;

// current rescue step
uint8_t _rescue_step_index = 0;

uint16_t rescue_get_timeout_seconds(void) {
    return _rescue_timeout_seconds;
}

void rescue_set_timeout_seconds(uint16_t _seconds) {
    _rescue_timeout_seconds = _seconds;
}

bool rescue_get_enable_status(void) {
    return _rescue_enabled;
}

void rescue_set_enabled(void) {
    _rescue_enabled = true;
}

void rescue_set_disabled(void) {
    _rescue_enabled = false;
}

bool rescue_get_fired_status(void) {
    return _rescue_fired;
}

void rescue_reset_fired_status(void) {
    _rescue_fired = false;
}

void _rescue_action_step_0(void) {
    // First rescue action:
    // Close VEOC direct switch
    switch_SWITCH_VEOC_DIRECT_set_closed();
}

void _rescue_action_step_1(void) {
    // Second rescue action:
    // Close VEOC return switch
    switch_SWITCH_VEOC_RETURN_set_closed();
}

void _rescue_action_step_2(void) {
    // Third rescue action:
    // Enable 12V DCDC
    switch_SWITCH_12V_set_closed();
}

void _rescue_action_step_3(void) {
    // Fourth rescue action:
    // Close HYDRO switch
    switch_SWITCH_HYDRO_set_closed();
}

void _rescue_action_step_4(void) {
    // Fifth rescue action:
    // Close LBL switch
    switch_SWITCH_LBL_set_closed();
}

// rescue action list
rescue_action_t _rescue_actions[RESCUE_ACTION_STEP_COUNT] = {
    _rescue_action_step_0,
    _rescue_action_step_1,
    _rescue_action_step_2,
    _rescue_action_step_3,
    _rescue_action_step_4
};

void rescue_check(void) {
    // check if rescue is enabled
    if(_rescue_enabled) {
        // check if timeout has elapsed
        if(timing_get_seconds_since_last_command() > (uint32_t)_rescue_timeout_seconds) {
            // signal rescue was started
            _rescue_fired = true;
            // check next action to perform
            if(_rescue_step_index < RESCUE_ACTION_STEP_COUNT) {
                if(timing_get_rescue_step_timeout() == 0) {
                    // perform current action
                    _rescue_actions[_rescue_step_index]();
                    // set timeout for next action                    
                    #if RESCUE_STEP_INTERVAL_SECONDS > 0xFF
                        #error "RESCUE_STEP_INTERVAL_SECONDS too big for uint8_t"
                    #endif
                    timing_set_rescue_step_timeout(RESCUE_STEP_INTERVAL_SECONDS);
                    // increment action step index for next call
                    _rescue_step_index += 1;
                }                
            }
        }else{
            // if timeout not elapsed, reset state machine
            _rescue_step_index = 0;
            timing_set_rescue_step_timeout(0);
        }
    }
}